/* 
  Copyright 1991 Mario Goertzel and Matthew Sorrels
 
  Permission to use, copy, modify, distribute, and sell this software and its
  documentation for any purpose is hereby granted without fee, provided that
  the above copyright notice appear in all copies and that both that
  copyright notice and this permission notice appear in supporting
  documentation, and that the name of the author not be used in advertising or
  publicity pertaining to distribution of the software without specific,
  written prior permission.  The authors make no representations about the
  suitability of this software for any purpose.  It is provided "as is"
  without express or implied warranty.
 
  MARIO GOERTZEL AND MATTHEW SORRELS DISCLAIMS ALL WARRANTIES WITH 
  REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF 
  MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL MATTHEW SORRELS BE 
  LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY 
  DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
  Authors:  Mario Goertzel and Matthew Sorrels
*/

/*
 * Capture The Flag
 *
 * Mario Goertzel <mg2p+@andrew.cmu.edu>
 * Matthew Sorrels <ms90+@andrew.cmu.edu>
 *
 * May 11, 1991
 */
#include <stdio.h>
#include <strings.h>
#include <sys/param.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include "capture.h"
#include "CaptureIcon.xbm"

struct Board mainBoard;
struct Piece pieces1[17];
struct Piece pieces2[17];
char *pieceNames[] = {
    "Marshal", "General", "Colonel",
    "Major", "Captain", "Lieutenant",
    "Sergeant", "Miner", "Scout",
    "Spy", "Bomb", "Flag",
    "Unknown", "LakeA", "LakeB", "Victory1", "Victory2"
};

#define PIECES 40
int pieceCount[] = {
  1, 1, 2,
  3, 4, 4,
  4, 5, 8,
  1, 6, 1,
  0, 0, 0,
  0, 0
};

int finalwinner;
int turnnum;
int turn;
int setup1;
int setup2;

char *StatusText[2];
char *Player1Text;
char *Player2Text;
int TextSize[2];

int windowX = 20,
    windowY = 20;

char *DisplayName1 = NULL;
char *DisplayName2 = NULL;

Display *Display1, *Display2;
int Screen1, Screen2;
int Depth1, Depth2;
Colormap Colormap1, Colormap2;
unsigned long BackColor1, BackColor2;
unsigned long ForeColor1, ForeColor2;
unsigned long PieceColor1, PieceColor2;
unsigned long OppColor1, OppColor2;

Window wind1, wind2;
Window board1, board2;
Window status1, status2;
Window talk1, talk2;
Window otherTalk1, otherTalk2;
GC draw1, draw2, erase1, erase2, xor1, xor2, invert1, invert2;  
XFontStruct *font1, *font2;
Cursor cursor1, cursor2;
char *BaseDir = BASEDIRNAME;
char *FontName = "fixed";
int textwidth1, textwidth2;
int textheight1, textheight2;
int BorderWidth = 2;

int Isinvert1 = 0, Isinvert2 = 0;
int InvertI1, InvertJ1;
int InvertI2, InvertJ2;

#define MAIN_WIDTH 602
#define MAIN_HEIGHT 608

main(argc, argv)
int argc;
char *argv[];
{
    int count;
    unsigned int userX, userY, userWidth, userHeight, GeomStatus;

    for(count=1;count<argc;count++)
      {
	  if ((argc - count) > 1)
	    {
		if (!strcmp(argv[count],"-geometry"))
		  {
		      GeomStatus = XParseGeometry(argv[count+1],&userX,&userY,&userWidth,&userHeight);
		      if (GeomStatus & XValue)
			windowX = userX;
		      if (GeomStatus & YValue)
			windowY = userY;
		      count ++;
		      continue;
		  }
		if (!strcmp(argv[count],"-display"))
		  {
		      DisplayName1 = (char *)malloc(strlen(argv[count+1]) + 1);
                      strcpy(DisplayName1,argv[count+1]);
                      count++;
                      continue;
		  }
		if (!strcmp(argv[count],"-bitmaps"))
		  {
		      BaseDir = (char *)malloc(strlen(argv[count+1]) + 1);
                      strcpy(BaseDir,argv[count+1]);
                      count++;
                      continue;
		  }		
	    }

	  if ((!strcmp(argv[count],"-help")) ||
	      (DisplayName2 != NULL))
	    {
		fprintf(stderr,"Usage: %s <2nd-Player Display>\n",argv[0]);
		fprintf(stderr,"\t-help                        This Message\n");
		fprintf(stderr,"\t-display  display_name       Player 1 Display\n");
		fprintf(stderr,"\t-geometry [{width}][x{height}][{+-}{xoff}[{+-}{yoff}]]]\n");
		fprintf(stderr,"\t-bitmaps directory_name      Base to piece bitmaps\n\n");
		fprintf(stderr,"During game:  Ctrl-q     Quits\n");
		fprintf(stderr,"              Ctrl-n     Starts A New Game\n");
		fprintf(stderr,"              Ctrl-l     Refresh Screen\n");
		exit(-1);
	    }
	  DisplayName2 = (char *)malloc(strlen(argv[count]) + 1);
	  strcpy(DisplayName2,argv[count]);
      }
    CreateDisplays();
    CreateWindows();
    DoEvents();
}

CreateDisplays()
{
    Display1 = XOpenDisplay(DisplayName1);
    if (Display1 == NULL)
      {
	  fprintf(stderr,"Can't open connection to Player 1 Xserver\n");
	  exit(-1);
      }
    Display2 = XOpenDisplay(DisplayName2);
    if (Display2 == NULL)
      {
	  fprintf(stderr,"Can't open connection to Player 2 Xserver\n");
	  exit(-1);
      }
    Screen1 = DefaultScreen(Display1);
    Screen2 = DefaultScreen(Display2);
    Depth1 = DefaultDepth(Display1,Screen1);
    Depth2 = DefaultDepth(Display2,Screen2);
    Colormap1 = DefaultColormap(Display1,Screen1);
    Colormap2 = DefaultColormap(Display2,Screen2);    
    BackColor1 = WhitePixel(Display1,Screen1);
    BackColor2 = WhitePixel(Display2,Screen2);
    ForeColor1 = BlackPixel(Display1,Screen1);
    ForeColor2 = BlackPixel(Display2,Screen2);
    PieceColor1 = BlackPixel(Display1,Screen1);
    OppColor1 = BlackPixel(Display1,Screen1);
    PieceColor2 = BlackPixel(Display2,Screen2);
    OppColor2 = BlackPixel(Display2,Screen2);
}
    
ReadPieces()
{
    char bitmapFile[MAXPATHLEN+1];
    int i, status, hot;

    for(i=0;i<17;i++)
      {
	  sprintf(bitmapFile,"%s/%s.xbm",BaseDir,pieceNames[i]);
	  status = XReadBitmapFile(Display1, wind1, bitmapFile,
				   &(pieces1[i].width), &(pieces1[i].height),
				   &(pieces1[i].icon), &hot, &hot);
	  if (status != BitmapSuccess)
	    {
		fprintf(stderr,"Unable to read bitmap file %s\n",
			bitmapFile);
		exit(-1);
	    }
	  status = XReadBitmapFile(Display2, wind2, bitmapFile,
				   &(pieces2[i].width), &(pieces2[i].height),
				   &(pieces2[i].icon), &hot, &hot);
	  if (status != BitmapSuccess)
	    {
		fprintf(stderr,"Unable to read bitmap file %s\n",
			bitmapFile);
		exit(-1);
	    }
      }
}

CreateWindows()
{
    XSetWindowAttributes thewindatt;
    XGCValues GCValues;
    XSizeHints SizeHints;
    unsigned long windmask, GCMask;
    Pixmap icon1, icon2;
    int i;

    cursor1 = XCreateFontCursor(Display1, XC_left_ptr);
    cursor2 = XCreateFontCursor(Display2, XC_left_ptr);
    font1 = XLoadQueryFont(Display1,FontName);
    font2 = XLoadQueryFont(Display2,FontName);
    if (font1 == NULL || font2 == NULL)
      {
	  fprintf(stderr,"Unable to allocate font %s\n",FontName);
	  exit(-1);
      }
    textheight1 = font1->ascent + font1->descent;
    textheight2 = font2->ascent + font2->descent;
    TextSize[0] = 600 / (font1->max_bounds.rbearing - font1->min_bounds.lbearing);
    TextSize[1] = 600 / (font2->max_bounds.rbearing - font2->min_bounds.lbearing);
    StatusText[0] = (char *)malloc(TextSize[0]+1);
    StatusText[1] = (char *)malloc(TextSize[1]+1);
    *(StatusText[0])= '\0';
    *(StatusText[1]) = '\0';
    Player1Text = (char *)malloc(TextSize[0]+1);
    Player2Text = (char *)malloc(TextSize[1]+1);
    *(Player1Text) = '\0';
    *(Player2Text) = '\0';    
    thewindatt.background_pixel = BackColor1;
    thewindatt.cursor = cursor1;
    thewindatt.backing_store = True;
    windmask = (CWBackPixel | CWCursor | CWBackingStore);
    wind1 = XCreateWindow(Display1, RootWindow(Display1, Screen1),
			  windowX, windowY, MAIN_WIDTH,
			  MAIN_HEIGHT + 3*textheight1,
			  BorderWidth, Depth1, InputOutput, CopyFromParent,
			  windmask, &thewindatt);
    GCValues.foreground = ForeColor1;
    GCValues.background = BackColor1;
    GCValues.line_width = 1;
    GCValues.font = font1->fid;
    GCValues.function = GXcopy;
    GCMask = (GCFunction | GCForeground | GCBackground | GCLineWidth |GCFont);
    draw1 = XCreateGC(Display1, wind1, GCMask, &GCValues);
    GCValues.function = GXclear;
    erase1 = XCreateGC(Display1, wind1, GCMask, &GCValues);
    GCValues.function = GXequiv;
    xor1 = XCreateGC(Display1, wind1, GCMask, &GCValues);    
    GCValues.function = GXcopyInverted;
    invert1 = XCreateGC(Display1, wind1, GCMask, &GCValues);    
    board1 = XCreateWindow(Display1, wind1,0,0,600,600,1,Depth1,InputOutput,
			   CopyFromParent,windmask,&thewindatt);
    thewindatt.background_pixel = ForeColor1;
    status1 = XCreateWindow(Display1, wind1,0,602,600,textheight1,1,
			    Depth1,InputOutput,CopyFromParent,
			    windmask,&thewindatt);
    thewindatt.background_pixel = BackColor1;
    otherTalk1 = XCreateWindow(Display1, wind1,0,604+textheight1,600,
			  textheight1,1,Depth1,InputOutput,
			  CopyFromParent,windmask,&thewindatt);    
    talk1 = XCreateWindow(Display1, wind1,0,606+2*textheight1,600,
			  textheight1,1,Depth1,InputOutput,
			  CopyFromParent,windmask,&thewindatt);    
    thewindatt.cursor = cursor2;
    thewindatt.background_pixel = BackColor2;
    wind2 = XCreateWindow(Display2, RootWindow(Display2, Screen2),
			  windowX, windowY, MAIN_WIDTH,
			  MAIN_HEIGHT + 3*textheight2,
			  BorderWidth, Depth2, InputOutput, CopyFromParent,
			  windmask, &thewindatt);
    GCValues.foreground = ForeColor2;
    GCValues.background = BackColor2;
    GCValues.line_width = 1;
    GCValues.font = font2->fid;
    GCValues.function = GXcopy;
    GCMask = (GCFunction | GCForeground | GCBackground | GCLineWidth |GCFont);
    draw2 = XCreateGC(Display2, wind2, GCMask, &GCValues);
    GCValues.function = GXclear;
    erase2 = XCreateGC(Display2, wind2, GCMask, &GCValues);
    GCValues.function = GXequiv;
    xor2 = XCreateGC(Display2, wind2, GCMask, &GCValues);    
    GCValues.function = GXcopyInverted;
    invert2 = XCreateGC(Display2, wind2, GCMask, &GCValues);    
    board2 = XCreateWindow(Display2, wind2,0,0,600,600,1,Depth2,InputOutput,
			   CopyFromParent,windmask,&thewindatt);
    thewindatt.background_pixel = ForeColor2;    
    status2 = XCreateWindow(Display2, wind2,0,602,600,textheight2,1,
			    Depth2,InputOutput,CopyFromParent,
			    windmask,&thewindatt);
    thewindatt.background_pixel = BackColor2;    
    otherTalk2 = XCreateWindow(Display2, wind2,0,604+textheight2,600,
			  textheight2,1,Depth2,InputOutput,
			  CopyFromParent,windmask,&thewindatt);    
    talk2 = XCreateWindow(Display2, wind2,0,606+2*textheight2,600,
			  textheight2,1,Depth2,InputOutput,
			  CopyFromParent,windmask,&thewindatt);    

    icon1 = XCreateBitmapFromData(Display1,wind1, CaptureIcon_bits, CaptureIcon_width, CaptureIcon_height);
    icon2 = XCreateBitmapFromData(Display2,wind2, CaptureIcon_bits, CaptureIcon_width, CaptureIcon_height);

    SizeHints.min_width = SizeHints.max_width = MAIN_WIDTH;
    SizeHints.min_height = SizeHints.max_height = MAIN_HEIGHT + 3*textheight1;
    SizeHints.flags = (PSize | PMaxSize | PMinSize);
    /* The X Consortium can bite me...A string is a (char *) */
    XSetStandardProperties(Display1, wind1, "Capture The Flag","Capture",icon1,NULL,0,&SizeHints);
    SizeHints.min_height = SizeHints.max_height = MAIN_HEIGHT + 3*textheight2;
    XSetStandardProperties(Display2, wind2, "Capture The Flag","Capture",icon2,NULL,0,&SizeHints);

    ReadPieces();

    XMapWindow(Display1, wind1);
    XMapSubwindows(Display1, wind1);
    XMapWindow(Display2, wind2);
    XMapSubwindows(Display2, wind2);    
    XFlush(Display1);
    XFlush(Display2);
}


InitBoard()
{
  int i, j, k;
  int pieces1[PIECES];
  int pieces2[PIECES];

  /* Center two rows */
  for(i = 4; i<6; i++){
    for(j = 0; j<2; j++){
      mainBoard.location[j][i].status = OPEN;
      mainBoard.location[j][i].value = EMPTY;
      mainBoard.location[j+2][i].status = BLOCKED;
      mainBoard.location[j+2][i].value = LAKEA;
      mainBoard.location[j+4][i].status = OPEN;
      mainBoard.location[j+4][i].value = EMPTY;
      mainBoard.location[j+6][i].status = BLOCKED;
      mainBoard.location[j+6][i].value = LAKEB;
      mainBoard.location[j+8][i].status = OPEN;
      mainBoard.location[j+8][i].value = EMPTY;
    }
  }

  for(k=0, i=0; i<17; i++){
    for(j=0; j < pieceCount[i]; j++){
      pieces1[k] = i;
      pieces2[k] = i;
      k++;
    }
  }

  srandom(time(0));

  for(i=0; i<10; i++){
    for(j=0; j<4; j++){
      int l;
      l = (int) random() % k;
      mainBoard.location[i][j].status = PLAYER1;
      mainBoard.location[i][j].value = pieces1[l];
      pieces1[l] = pieces1[k-1];

      l = (int) random() % k;
      mainBoard.location[i][j+6].status = PLAYER2;
      mainBoard.location[i][j+6].value = pieces2[l];
      k--;
      pieces2[l] = pieces2[k];
    }
  }
}
      
RedrawBoard(boardnum)
int boardnum;
{
  int i, j, k;
  Display *this_dis;
  Window board, statwin, otherwin, talkwin;
  GC invert, draw, xor, tmpdraw;
  struct Piece *pieces;
  int textheight;
  char *MainText, *OtherText;

  if (boardnum == 1)
    {
	this_dis = Display1;
	board = board1;
	invert = invert1;
	draw = draw1;
	xor = xor1;
	pieces = pieces1;
	statwin = status1;
	otherwin = otherTalk1;
	talkwin = talk1;
	textheight = textheight1 - font2->descent;
	MainText = Player1Text;
	OtherText = Player2Text;
    }
  else
    {
	this_dis = Display2;
	board = board2;
	invert = invert2;
	draw = draw2;
	xor = xor2;
	pieces = pieces2;
	statwin = status2;
	otherwin = otherTalk2;
	talkwin = talk2;
	textheight = textheight2 - font2->descent;
	MainText = Player2Text;
	OtherText = Player1Text;	
    }
  for(i=0;i<600;i+=60)
    {
	XDrawLine(this_dis,board,draw,i,0,i,600);
	XDrawLine(this_dis,board,draw,0,i,600,i);
    }
  if (!finalwinner)
    {
  XCopyPlane(this_dis,pieces[LAKEA].icon,board,draw,0,0,
	     pieces[LAKEA].width,pieces[LAKEA].height,
	     121,241,1);
  XCopyPlane(this_dis,pieces[LAKEB].icon,board,draw,0,0,
	     pieces[LAKEA].width,pieces[LAKEA].height,
	     361,241,1);
    }
  for(i=0; i<10; i++) {
      for(j=0; j<10; j++) {
	  k = mainBoard.location[i][j].value;
	  if (k == LAKEA || k == LAKEB || k == EMPTY) continue;
	  tmpdraw = draw;
	  if (mainBoard.location[i][j].status != boardnum)
	    {
		if (finalwinner)
		  tmpdraw = invert;
		else
		  k = UNKNOWN;
	    }
	  XCopyPlane(this_dis, pieces[k].icon, board,tmpdraw, 0, 0,
		     pieces[k].width, pieces[k].height,
		     i*60+7, j*60+3, 1);
      }
  }
  if (boardnum == 1 && Isinvert1 && (mainBoard.location[InvertI1][InvertJ1].status == boardnum))
    {
	k = mainBoard.location[InvertI1][InvertJ1].value;
	XCopyPlane(this_dis, pieces[k].icon, board,invert, 0, 0,
		   pieces[k].width, pieces[k].height,
		   InvertI1*60+7, InvertJ1*60+3, 1);
    }
  if (boardnum == 2 && Isinvert2 && (mainBoard.location[InvertI2][InvertJ2].status == boardnum)) 
    {
	k = mainBoard.location[InvertI2][InvertJ2].value;
	XCopyPlane(this_dis, pieces[k].icon, board,invert, 0, 0,
		   pieces[k].width, pieces[k].height,
		   InvertI2*60+7, InvertJ2*60+3, 1);
    }
  XDrawString(this_dis,statwin,invert,1,textheight,StatusText[boardnum-1],strlen(StatusText[boardnum-1]));
  XDrawString(this_dis,otherwin,draw,1,textheight,OtherText ,strlen(OtherText));
  XDrawString(this_dis,talkwin,draw,1,textheight,MainText,strlen(MainText));
  if (finalwinner) Victory(finalwinner);
  XFlush(this_dis);
}

DrawTextBox(boxnum,clear)
int boxnum;
int clear;
{
    switch(boxnum)
      {
      case 0:
	  if (clear)
	    {
		XClearWindow(Display1,status1);
		XClearWindow(Display2,status2);
	    }
	  XDrawString(Display1,status1,invert1,1,font1->ascent,StatusText[0],strlen(StatusText[0]));	  
	  XDrawString(Display2,status2,invert2,1,font2->ascent,StatusText[1],strlen(StatusText[1]));
	  XFlush(Display1);
	  XFlush(Display2);
	  break;
      case 1:
	  if (clear)
	    {
		XClearWindow(Display1,talk1);
		XClearWindow(Display2,otherTalk2);
	    }
	  XDrawString(Display1,talk1,draw1,1,font1->ascent,Player1Text,strlen(Player1Text));	  
	  XDrawString(Display2,otherTalk2,draw2,1,font2->ascent,Player1Text,strlen(Player1Text));
	  XFlush(Display1);
	  XFlush(Display2);
	  break;
      case 2:
	  if (clear)
	    {
		XClearWindow(Display1,otherTalk1);
		XClearWindow(Display2,talk2);
	    }
	  XDrawString(Display1,otherTalk1,draw1,1,font1->ascent,Player2Text,strlen(Player2Text));	  
	  XDrawString(Display2,talk2,draw2,1,font2->ascent,Player2Text,strlen(Player2Text));
	  XFlush(Display1);
	  XFlush(Display2);
	  break;	  
      }
}

InvertPiece(player, i,j)
int player;
int i,j;
{
    int k;
    int deinvert = 0;
    GC useme;
    
    if (player == 1)
      {
	  useme = invert1;
	  if (mainBoard.location[i][j].status != player) return;
	  if ((Isinvert1) && (InvertI1 == i) && (InvertJ1 == j))
	    {
		useme = draw1;
		Isinvert1 = 0;
	    }
	  else
	    Isinvert1 = 1;
	  InvertI1 = i;
	  InvertJ1 = j;
	  XClearArea(Display1,board1,InvertI1*60+1, InvertJ1*60+1,59,59,0);
	  k = mainBoard.location[InvertI1][InvertJ1].value;
	  XCopyPlane(Display1, pieces1[k].icon, board1,useme, 0, 0,
		   pieces1[k].width, pieces1[k].height,
		   InvertI1*60+7, InvertJ1*60+3, 1);
	  XFlush(Display1);
  
      }
    else
      {
	  useme = invert2;
	  if (mainBoard.location[i][j].status != player) return;
	  if ((Isinvert2) && (InvertI2 == i) && (InvertJ2 == j))
	    {
		useme = draw2;
		Isinvert2 = 0;
	    }
	  else
	    Isinvert2 = 1;	  
	  InvertI2 = i;
	  InvertJ2 = j;
	  XClearArea(Display2,board2,InvertI2*60+1, InvertJ2*60+1,59,59,0);
	  k = mainBoard.location[InvertI2][InvertJ2].value;
	  XCopyPlane(Display2, pieces2[k].icon, board2,useme, 0, 0,
		   pieces2[k].width, pieces2[k].height,
		   InvertI2*60+7, InvertJ2*60+3, 1);
	  XFlush(Display2);
      }
}

ExchangePieces(player, i, j)
int player;
int i,j;
{
    int k, q;
    char tmpstat, tmpvalue;
    
    if (player == 1)
      {
	  if ((mainBoard.location[i][j].status != player) ||
	      !Isinvert1) return;
	  Isinvert1 = 0;
	  XClearArea(Display1,board1,InvertI1*60+1, InvertJ1*60+1,59,59,0);
	  XClearArea(Display1,board1,i*60+1, j*60+1,59,59,0);
	  tmpstat =mainBoard.location[InvertI1][InvertJ1].status;
	  tmpvalue =mainBoard.location[InvertI1][InvertJ1].value;
	  mainBoard.location[InvertI1][InvertJ1].status =
	    mainBoard.location[i][j].status;
	  mainBoard.location[InvertI1][InvertJ1].value =
	    mainBoard.location[i][j].value;
	  mainBoard.location[i][j].status = tmpstat;
	  mainBoard.location[i][j].value = tmpvalue;
	  k = mainBoard.location[InvertI1][InvertJ1].value;
	  q = mainBoard.location[i][j].value;	  
	  XCopyPlane(Display1, pieces1[k].icon, board1,draw1, 0, 0,
		   pieces1[k].width, pieces1[k].height,
		   InvertI1*60+7, InvertJ1*60+3, 1);
	  XCopyPlane(Display1, pieces1[q].icon, board1,draw1, 0, 0,
		   pieces1[q].width, pieces1[q].height,
		   i*60+7, j*60+3, 1);
	  XFlush(Display1);
      }
    else
      {
	  if ((mainBoard.location[i][j].status != player) ||
	      !Isinvert2) return;
	  Isinvert2 = 0;
	  XClearArea(Display2,board2,InvertI2*60+1, InvertJ2*60+1,59,59,0);
	  XClearArea(Display2,board2,i*60+1, j*60+1,59,59,0);
	  tmpstat =mainBoard.location[InvertI2][InvertJ2].status;
	  tmpvalue =mainBoard.location[InvertI2][InvertJ2].value;
	  mainBoard.location[InvertI2][InvertJ2].status =
	    mainBoard.location[i][j].status;
	  mainBoard.location[InvertI2][InvertJ2].value =
	    mainBoard.location[i][j].value;
	  mainBoard.location[i][j].status = tmpstat;
	  mainBoard.location[i][j].value = tmpvalue;
	  k = mainBoard.location[InvertI2][InvertJ2].value;
	  q = mainBoard.location[i][j].value;	  
	  XCopyPlane(Display2, pieces2[k].icon, board2,draw2, 0, 0,
		   pieces2[k].width, pieces2[k].height,
		   InvertI2*60+7, InvertJ2*60+3, 1);
	  XCopyPlane(Display2, pieces2[q].icon, board2,draw2, 0, 0,
		   pieces2[q].width, pieces2[q].height,
		   i*60+7, j*60+3, 1);
	  XFlush(Display2);
      }
}
  
AnimatePiece(i1,j1,i2,j2)
int i1, j1, i2, j2;
{
    int value, value1, value2, status;
    int x,y,start,stop,direction, axis;

    value = mainBoard.location[i1][j1].value;
    status = mainBoard.location[i1][j1].status;
    value1 = (status == PLAYER1 ? value : UNKNOWN);
    value2 = (status == PLAYER2 ? value : UNKNOWN);

    if (i1 == i2)
      {
	  axis = 0;
	  start = j1*60+3;
	  stop = j2*60+3;
	  if (start > stop)
	    direction = -5;
	  else
	    direction = 5;
      }
    else
      {
	  axis = 1;
	  start = i1*60+7;
	  stop = i2*60+7;
	  if (start > stop)
	    direction = -5;
	  else
	    direction = 5;
      }
    x = i1*60+7;
    y = j1*60+3;
    XClearArea(Display1,board1,i1*60+1,j1*60+1,59,59,0);
    XClearArea(Display2,board2,i1*60+1,j1*60+1,59,59,0);
    if (status == PLAYER1)
      Isinvert1 = 0;
    else
      Isinvert2 = 0;
    XFlush(Display1);
    XFlush(Display2);
    for(;start!=stop;start+=direction)
      {
	  if (axis)
	    x = start;
	  else
	    y = start;
	  XCopyPlane(Display1, pieces1[value1].icon, board1,xor1, 0, 0,
		   pieces1[value1].width, pieces1[value1].height,
		   x, y, 1);
	  XCopyPlane(Display2, pieces2[value2].icon, board2,xor2, 0, 0,
		     pieces2[value2].width, pieces2[value2].height,
		     x, y, 1);
	  XFlush(Display1);
	  XFlush(Display2);
	  uwait(0,5000); 
	  XCopyPlane(Display1, pieces1[value1].icon, board1,xor1, 0, 0,
		   pieces1[value1].width, pieces1[value1].height,
		   x, y, 1);
	  XCopyPlane(Display2, pieces2[value2].icon, board2,xor2, 0, 0,
		     pieces2[value2].width, pieces2[value2].height,
		     x, y, 1);
	  XFlush(Display1);
	  XFlush(Display2);
      }
    XCopyPlane(Display1, pieces1[value1].icon, board1,draw1, 0, 0,
	       pieces1[value1].width, pieces1[value1].height,
	       i2*60+7, j2*60+3, 1);
    XCopyPlane(Display2, pieces2[value2].icon, board2,draw2, 0, 0,
	       pieces2[value2].width, pieces2[value2].height,
	       i2*60+7, j2*60+3, 1);
    XFlush(Display1);
    XFlush(Display2);
    mainBoard.location[i1][j1].value = EMPTY;
    mainBoard.location[i1][j1].status = OPEN;
    mainBoard.location[i2][j2].value = value;
    mainBoard.location[i2][j2].status = status;
}

        
    
Strike(i1, j1, i2, j2)
int i1, j1, i2, j2;
{
    Pixmap old1, old2;
    int k, q, win;
    int k1, k2;

    XClearArea(Display1,board1,i1*60+1,j1*60+1,59,59,0);
    XClearArea(Display2,board2,i1*60+1,j1*60+1,59,59,0);
    XClearArea(Display1,board1,i2*60+1,j2*60+1,59,59,0);
    XClearArea(Display2,board2,i2*60+1,j2*60+1,59,59,0);
    old1 = XCreatePixmap(Display1,board1,120,120,Depth1);
    old2 = XCreatePixmap(Display2,board2,120,120,Depth2);
    XCopyArea(Display1, board1, old1, draw1, i2*60,j2*60,120,120,0,0);
    XCopyArea(Display2, board2, old2, draw2, i2*60,j2*60,120,120,0,0);
    k = mainBoard.location[i1][j1].value;
    q = mainBoard.location[i2][j2].value;
    XCopyPlane(Display1,pieces1[q].icon,board1,draw1,0,0,
	       pieces1[q].width, pieces1[q].height,
	       i2*60+7,j2*60+3,1);
    XCopyPlane(Display2,pieces2[q].icon,board2,draw2,0,0,
	       pieces2[q].width, pieces2[q].height,
	       i2*60+7,j2*60+3,1);
    XCopyPlane(Display1,pieces1[k].icon,board1,invert1,0,0,
	       pieces1[k].width, pieces1[k].height,
	       i2*60+7+24,j2*60+3+28,1);
    XCopyPlane(Display2,pieces2[k].icon,board2,invert2,0,0,
	       pieces2[k].width, pieces2[k].height,
	       i2*60+7+24,j2*60+3+28,1);
    XFlush(Display1);
    XFlush(Display2);
    uwait(2,0);
    XCopyArea(Display1,old1,board1,draw1,0,0,120,120,i2*60,j2*60);
    XCopyArea(Display2,old2,board2,draw2,0,0,120,120,i2*60,j2*60);
    win = Winner(i1,j1,i2,j2);
    switch(win)
      {
      case 0:
    	  mainBoard.location[i2][j2].value = mainBoard.location[i1][j1].value;
	  mainBoard.location[i2][j2].status = mainBoard.location[i1][j1].status;
	  mainBoard.location[i1][j1].value = EMPTY;
	  mainBoard.location[i1][j1].status = OPEN;
	  k1 = k2 = mainBoard.location[i2][j2].value;
	  if (mainBoard.location[i2][j2].status == PLAYER1)
	    k2 = UNKNOWN;
	  else
	    k1 = UNKNOWN;	    
	  XCopyPlane(Display1,pieces1[k1].icon,board1,draw1,0,0,
		     pieces1[k1].width, pieces1[k1].height,
		     i2*60+7,j2*60+3,1);
	  XCopyPlane(Display2,pieces2[k2].icon,board2,draw2,0,0,
	       pieces2[k2].width, pieces2[k2].height,
	       i2*60+7,j2*60+3,1);
	  break;
      case 1:
	  mainBoard.location[i1][j1].value = mainBoard.location[i2][j2].value;
	  mainBoard.location[i1][j1].status = mainBoard.location[i2][j2].status;
	  mainBoard.location[i2][j2].value = EMPTY;
	  mainBoard.location[i2][j2].status = OPEN;
	  k1 = k2 = mainBoard.location[i1][j1].value;
	  if (mainBoard.location[i1][j1].status == PLAYER1)
	    k2 = UNKNOWN;
	  else
	    k1 = UNKNOWN;	    
	  XCopyPlane(Display1,pieces1[k1].icon,board1,draw1,0,0,
		     pieces1[k1].width, pieces1[k1].height,
		     i1*60+7,j1*60+3,1);
	  XCopyPlane(Display2,pieces2[k2].icon,board2,draw2,0,0,
	       pieces2[k2].width, pieces2[k2].height,
	       i1*60+7,j1*60+3,1);
	  break;
      case 2:
	  mainBoard.location[i1][j1].value = EMPTY;
	  mainBoard.location[i1][j1].status = OPEN;
	  mainBoard.location[i2][j2].value = EMPTY;
	  mainBoard.location[i2][j2].status = OPEN;
	  break;
      case 3:
	  mainBoard.location[i1][j1].value = EMPTY;
	  mainBoard.location[i1][j1].status = OPEN;
	  k1 = k2 = mainBoard.location[i2][j2].value;
	  if (mainBoard.location[i2][j2].status == PLAYER1)
	    k2 = UNKNOWN;
	  else
	    k1 = UNKNOWN;	    
	  XCopyPlane(Display1,pieces1[k1].icon,board1,draw1,0,0,
		     pieces1[k1].width, pieces1[k1].height,
		     i2*60+7,j2*60+3,1);
	  XCopyPlane(Display2,pieces2[k2].icon,board2,draw2,0,0,
	       pieces2[k2].width, pieces2[k2].height,
	       i2*60+7,j2*60+3,1);
	  break;	  
      }
    XFreePixmap(Display1, old1);
    XFreePixmap(Display2, old2);
}


Victory(whowon)
int whowon;
{
    XCopyPlane(Display1,pieces1[whowon-1+VICTORY1].icon,board1,draw1,0,0,
	       pieces1[whowon-1+VICTORY1].width,
	       pieces1[whowon-1+VICTORY1].height,
	       114, 241,1);
    XCopyPlane(Display2,pieces2[whowon-1+VICTORY1].icon,board2,draw2,0,0,
	       pieces2[whowon-1+VICTORY1].width,
	       pieces2[whowon-1+VICTORY1].height,
	       114, 241,1);
}
	       
